@@ -39,6 +39,7 @@ typedef uint64_t cipher_t;
 #define ACVP_CIPHERTYPE_MAC	0x0100000000000000ULL
 #define ACVP_CIPHERTYPE_ECC	0x0200000000000000ULL
 #define ACVP_CIPHERTYPE_DRBG	0x0400000000000000ULL
+#define ACVP_CIPHERTYPE_DOMAIN	0x0800000000000000ULL
 
 /* AES, TDES, AEAD, and SHA must be allowed to be ORed */
 
@@ -158,6 +159,20 @@ typedef uint64_t cipher_t;
 #define ACVP_DRBGHASH		(ACVP_CIPHERTYPE_DRBG | 0x0000000000000004ULL)
 #define ACVP_DRBGMASK		(ACVP_CIPHERTYPE_DRBG | 0x000000000000000fULL)
 
+#define ACVP_DH_MODP_2048	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000001ULL)
+#define ACVP_DH_MODP_3072	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000002ULL)
+#define ACVP_DH_MODP_4096	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000004ULL)
+#define ACVP_DH_MODP_6144	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000008ULL)
+#define ACVP_DH_MODP_8192	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000010ULL)
+#define ACVP_DH_FFDHE_2048	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000020ULL)
+#define ACVP_DH_FFDHE_3072	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000040ULL)
+#define ACVP_DH_FFDHE_4096	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000080ULL)
+#define ACVP_DH_FFDHE_6144	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000100ULL)
+#define ACVP_DH_FFDHE_8192	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000200ULL)
+#define ACVP_DH_FB		(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000400ULL)
+#define ACVP_DH_FC		(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000800ULL)
+#define ACVP_DH_DOMAIN_MASK	(ACVP_CIPHERTYPE_DOMAIN | 0x0000000000000fffULL)
+
 #define ACVP_UNKNOWN		0x0000000000000000ULL
 
 struct cipher_def_map {
@@ -258,6 +273,19 @@ static const struct cipher_def_map cipher_def_map[] = {
 	{ ACVP_CMAC_AES192, "CMAC-AES192" },
 	{ ACVP_CMAC_AES256, "CMAC-AES256" },
 	{ ACVP_CMAC_TDES, "CMAC-TDES" },
+
+	{ ACVP_DH_MODP_2048, "MODP-2048" },
+	{ ACVP_DH_MODP_3072, "MODP-3072" },
+	{ ACVP_DH_MODP_4096, "MODP-4096" },
+	{ ACVP_DH_MODP_6144, "MODP-6144" },
+	{ ACVP_DH_MODP_8192, "MODP-8192" },
+	{ ACVP_DH_FFDHE_2048, "ffdhe2048" },
+	{ ACVP_DH_FFDHE_3072, "ffdhe3072" },
+	{ ACVP_DH_FFDHE_4096, "ffdhe4096" },
+	{ ACVP_DH_FFDHE_6144, "ffdhe6144" },
+	{ ACVP_DH_FFDHE_8192, "ffdhe8192" },
+	{ ACVP_DH_FB, "FB" },
+	{ ACVP_DH_FC, "FC" },
 };
 
 #ifdef __cplusplus
