@@ -156,7 +156,7 @@ static const struct def_algo_prereqs aes_prereqs[] = {
 	{								\
 	OPENSSL_DRBG_CAPS_AES128,					\
 	.entropyinputlen = { 256, },					\
-	.noncelen = { DEF_ALG_ZERO_VALUE },				\
+	.noncelen = { 64 },						\
 	.df = false							\
 	}
 
@@ -178,7 +178,7 @@ static const struct def_algo_prereqs aes_prereqs[] = {
 	{								\
 	OPENSSL_DRBG_CAPS_AES192,					\
 	.entropyinputlen = { 320, },					\
-	.noncelen = { DEF_ALG_ZERO_VALUE, },				\
+	.noncelen = { 96, },						\
 	.df = false							\
 	}
 
@@ -200,7 +200,7 @@ static const struct def_algo_prereqs aes_prereqs[] = {
 	{								\
 	OPENSSL_DRBG_CAPS_AES256,					\
 	.entropyinputlen = { 384, },					\
-	.noncelen = { DEF_ALG_ZERO_VALUE, },				\
+	.noncelen = { 128, },						\
 	.df = false							\
 	}
 
@@ -405,6 +405,20 @@ static const struct def_algo_rsa_siggen_caps openssl_rsa_siggen_caps[] = { {
 	OPENSSL_RSA_SIGGEN_CAPS_COMMON
 } };
 
+static const struct def_algo_rsa_siggen_caps openssl_rsa_siggen_caps_pss[] = { {
+	.rsa_modulo = DEF_ALG_RSA_MODULO_2048,
+	.saltlen = DEF_ALG_RSA_PSS_SALT_ZERO,
+	OPENSSL_RSA_SIGGEN_CAPS_COMMON
+}, {
+	.rsa_modulo = DEF_ALG_RSA_MODULO_3072,
+	.saltlen = DEF_ALG_RSA_PSS_SALT_ZERO,
+	OPENSSL_RSA_SIGGEN_CAPS_COMMON
+}, {
+	.rsa_modulo = DEF_ALG_RSA_MODULO_4096,
+	.saltlen = DEF_ALG_RSA_PSS_SALT_ZERO,
+	OPENSSL_RSA_SIGGEN_CAPS_COMMON
+} };
+
 #define OPENSSL_RSA_SIGGEN_CAPS_X931					\
 	.hashalg = ACVP_SHA1 | ACVP_SHA256 | ACVP_SHA384 |		\
 		   ACVP_SHA512
@@ -426,8 +440,8 @@ static const struct def_algo_rsa_siggen openssl_rsa_siggen[] = { {
 	.capabilities_num = ARRAY_SIZE(openssl_rsa_siggen_caps),
 }, {
 	.sigtype = DEF_ALG_RSA_SIGTYPE_PSS,
-	.capabilities = openssl_rsa_siggen_caps,
-	.capabilities_num = ARRAY_SIZE(openssl_rsa_siggen_caps),
+	.capabilities = openssl_rsa_siggen_caps_pss,
+	.capabilities_num = ARRAY_SIZE(openssl_rsa_siggen_caps_pss),
 }, {
 	.sigtype = DEF_ALG_RSA_SIGTYPE_ANSIX931,
 	.capabilities = openssl_rsa_siggen_caps_x931,
@@ -720,6 +734,21 @@ static const struct def_algo_kas_ecc_cdh_component openssl_kas_ecc_cdh = {
 /**************************************************************************
  * FFC DH Definitions
  **************************************************************************/
+static const struct def_algo_prereqs openssl_ffc_prereqs[] = {
+	{
+		.algorithm = "SHA",
+		.valvalue = "same"
+	},
+	{
+		.algorithm = "DRBG",
+		.valvalue = "same"
+	},
+	{
+		.algorithm = "DSA",
+		.valvalue = "same"
+	},
+};
+
 static const struct def_algo_kas_ffc_nokdfnokc openssl_kas_ffc_nokdfnokc_fb = {
 	.kas_ffc_paramset = DEF_ALG_KAS_FFC_FB,
 	.hashalg = ACVP_SHA224
@@ -734,7 +763,7 @@ static const struct def_algo_kas_ffc_nokdfnokc openssl_kas_ffc_nokdfnokc_fc = {
 	{								\
 	.type = DEF_ALG_TYPE_KAS_FFC,					\
 	.algo.kas_ffc = {						\
-		DEF_PREREQS(openssl_rsa_prereqs),			\
+		DEF_PREREQS(openssl_ffc_prereqs),			\
 		.kas_ffc_function = DEF_ALG_KAS_FFC_FULLVAL,		\
 		.kas_ffc_schema = DEF_ALG_KAS_FFC_DH_EPHEM,		\
 		.kas_ffc_role = DEF_ALG_KAS_FFC_INITIATOR |		\
@@ -857,17 +886,21 @@ static const struct def_algo openssl_aes [] = {
 
 	OPENSSL_AES_CCM,
 
-	OPENSSL_CMAC_AES
+	OPENSSL_CMAC_AES,
+
+	/* Built-in DRBG in crypto/rand/ */
+	OPENSSL_DRBG_CTR,
 };
 
 static const struct def_algo openssl_gcm [] = {
-	OPENSSL_AES_ECB,
-
 	OPENSSL_AES_GCM,
-	OPENSSL_AES_GMAC,
+	//zero length data not supported by OpenSSL
+	//OPENSSL_AES_GMAC,
 	OPENSSL_AES_GCM_IIV,
+};
 
-	OPENSSL_DRBG_CTR
+static const struct def_algo openssl_ffcdh [] = {
+	OPENSSL_KAS_FFC,
 };
 
 static const struct def_algo openssl_sha [] = {
@@ -892,20 +925,19 @@ static const struct def_algo openssl_sha [] = {
 	OPENSSL_ECDSA_SIGGEN,
 	OPENSSL_ECDSA_SIGVER,
 
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224, ACVP_SHA224),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224, ACVP_SHA256),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224, ACVP_SHA384),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224, ACVP_SHA512),
+	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224,
+			   ACVP_SHA224 | ACVP_SHA256 | ACVP_SHA384 |
+			   ACVP_SHA512),
 
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_256, ACVP_SHA256),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_256, ACVP_SHA384),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_256, ACVP_SHA512),
+	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_256,
+			   ACVP_SHA256 | ACVP_SHA384 | ACVP_SHA512),
 
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_3072, DEF_ALG_DSA_N_256, ACVP_SHA256),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_3072, DEF_ALG_DSA_N_256, ACVP_SHA384),
-	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_3072, DEF_ALG_DSA_N_256, ACVP_SHA512),
+	/* DSA_generate_key */
+	OPENSSL_DSA_PQGGEN(DEF_ALG_DSA_L_3072, DEF_ALG_DSA_N_256,
+			   ACVP_SHA256 | ACVP_SHA384 | ACVP_SHA512),
 
-	OPENSSL_DSA_PQGVER(DEF_ALG_DSA_L_1024, DEF_ALG_DSA_N_160, ACVP_SHA1),
+	//TODO OpenSSL SLES does not have 1024 bits, RHEL has it
+	//OPENSSL_DSA_PQGVER(DEF_ALG_DSA_L_1024, DEF_ALG_DSA_N_160, ACVP_SHA1),
 	OPENSSL_DSA_PQGVER(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_224, ACVP_SHA224),
 	OPENSSL_DSA_PQGVER(DEF_ALG_DSA_L_2048, DEF_ALG_DSA_N_256, ACVP_SHA256),
 	OPENSSL_DSA_PQGVER(DEF_ALG_DSA_L_3072, DEF_ALG_DSA_N_256, ACVP_SHA256),
@@ -937,7 +969,6 @@ static const struct def_algo openssl_sha [] = {
 
 	OPENSSL_KDF,
 	OPENSSL_KAS_ECC,
-	OPENSSL_KAS_FFC,
 	OPENSSL_KAS_ECC_CDH,
 
 	OPENSSL_PBKDF(ACVP_SHA1 |
@@ -968,22 +999,9 @@ static const struct def_algo openssl_sha3 [] = {
 		      ACVP_SHA3_512),
 };
 
-static const struct def_algo openssl_10x_sym_drbg [] = {
-	OPENSSL_AES_ECB,
+static const struct def_algo openssl_10x_drbg [] = {
+	/* DRBG in crypto/fips/ */
 	OPENSSL_DRBG_CTR,
-};
-
-static const struct def_algo openssl_10x_hash_drbg [] = {
-	OPENSSL_SHA(ACVP_SHA1),
-	OPENSSL_SHA(ACVP_SHA224),
-	OPENSSL_SHA(ACVP_SHA256),
-	OPENSSL_SHA(ACVP_SHA384),
-	OPENSSL_SHA(ACVP_SHA512),
-	OPENSSL_HMAC(ACVP_HMACSHA1),
-	OPENSSL_HMAC(ACVP_HMACSHA2_224),
-	OPENSSL_HMAC(ACVP_HMACSHA2_256),
-	OPENSSL_HMAC(ACVP_HMACSHA2_384),
-	OPENSSL_HMAC(ACVP_HMACSHA2_512),
 	OPENSSL_DRBG_HMAC,
 	OPENSSL_DRBG_HASH,
 };
@@ -1152,6 +1170,12 @@ static struct def_algo_map openssl_algo_map [] = {
 		.processor = "X86",
 		.impl_name = "SHA3_ASM"
 	}, {
+	/* OpenSSL FFC DH implementation **************************************/
+		SET_IMPLEMENTATION(openssl_ffcdh),
+		.algo_name = "OpenSSL",
+		.processor = "",
+		.impl_name = "FFC_DH"
+	}, {
 
 	/*
 	 * OpenSSL 1.0.x upstream DRBG implementation that may have been
@@ -1159,40 +1183,10 @@ static struct def_algo_map openssl_algo_map [] = {
 	 * The different instances relate to the different implementations of
 	 * the underlying cipher
 	 **********************************************************************/
-		SET_IMPLEMENTATION(openssl_10x_sym_drbg),
+		SET_IMPLEMENTATION(openssl_10x_drbg),
 		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_AESNI"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_sym_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_AESASM"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_sym_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_BAES_CTASM"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_SHA_AVX2"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_SHA_AVX"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_SHA_SSSE3"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "X86",
-		.impl_name = "DRBG_10X_SHA_ASM"
+		.processor = "",
+		.impl_name = "DRBG_10X"
 	}, {
 
 	/* OpenSSL ARM64v8 Assembler implementation ***************************/
@@ -1200,11 +1194,6 @@ static struct def_algo_map openssl_algo_map [] = {
 		.algo_name = "OpenSSL",
 		.processor = "ARM64",
 		.impl_name = "SHA_ASM"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "ARM64",
-		.impl_name = "DRBG_10X_SHA_ASM"
 	}, {
 	/* OpenSSL ARM64v8 SHA3 assembler implementation **********************/
 		SET_IMPLEMENTATION(openssl_sha3),
@@ -1220,6 +1209,11 @@ static struct def_algo_map openssl_algo_map [] = {
 		.algo_name = "OpenSSL",
 		.processor = "ARM64",
 		.impl_name = "CE"
+	}, {
+		SET_IMPLEMENTATION(openssl_gcm),
+		.algo_name = "OpenSSL",
+		.processor = "ARM64",
+		.impl_name = "CE_GCM"
 	}, {
 	/* OpenSSL ARM64v8 NEON bit slicing implementation ********************
 	 * Note: we may execute more ciphers than strictly provided by the NEON
@@ -1229,6 +1223,11 @@ static struct def_algo_map openssl_algo_map [] = {
 		.algo_name = "OpenSSL",
 		.processor = "ARM64",
 		.impl_name = "VPAES"
+	}, {
+		SET_IMPLEMENTATION(openssl_gcm),
+		.algo_name = "OpenSSL",
+		.processor = "ARM64",
+		.impl_name = "VPAES_GCM"
 	}, {
 	/* OpenSSL S390x Assembler implementation *****************************
 	 * Note: we may execute more ciphers than strictly provided by the ASM
@@ -1238,11 +1237,6 @@ static struct def_algo_map openssl_algo_map [] = {
 		.algo_name = "OpenSSL",
 		.processor = "S390",
 		.impl_name = "SHA_ASM"
-	}, {
-		SET_IMPLEMENTATION(openssl_10x_hash_drbg),
-		.algo_name = "OpenSSL",
-		.processor = "S390",
-		.impl_name = "DRBG_10X_SHA_ASM"
 	}, {
 	/* OpenSSL S390x SHA3 assembler implementation ************************/
 		SET_IMPLEMENTATION(openssl_sha3),
@@ -1255,6 +1249,11 @@ static struct def_algo_map openssl_algo_map [] = {
 		.algo_name = "OpenSSL",
 		.processor = "S390",
 		.impl_name = "AESASM"
+	}, {
+		SET_IMPLEMENTATION(openssl_gcm),
+		.algo_name = "OpenSSL",
+		.processor = "S390",
+		.impl_name = "AESASM_ASM"
 
 	},
 };
