@@ -30,6 +30,7 @@
 #include "request_helper.h"
 
 int acvp_req_set_prereq_pbkdf(const struct def_algo_pbkdf *pbkdf,
+			      const struct acvp_test_deps *deps,
 			      struct json_object *entry, bool publish)
 {
 	int ret;
@@ -40,7 +41,32 @@ int acvp_req_set_prereq_pbkdf(const struct def_algo_pbkdf *pbkdf,
 				     json_object_new_string("pbkdf")));
 
 	CKINT(acvp_req_gen_prereq(pbkdf->prereqvals,
-				  pbkdf->prereqvals_num, entry, publish));
+				  pbkdf->prereqvals_num, deps, entry, publish));
+
+out:
+	return ret;
+}
+
+int acvp_list_algo_pbkdf(const struct def_algo_pbkdf *pbkdf,
+			 struct acvp_list_ciphers **new)
+{
+	struct acvp_list_ciphers *tmp = NULL;
+	int ret;
+
+	(void)pbkdf;
+
+	tmp = calloc(1, sizeof(struct acvp_list_ciphers));
+	CKNULL(tmp, -ENOMEM);
+	*new = tmp;
+
+	CKINT(acvp_duplicate(&tmp->cipher_name, "kdf-components"));
+	CKINT(acvp_duplicate(&tmp->cipher_mode, "pbkdf"));
+	CKINT(acvp_req_cipher_to_stringarray(pbkdf->hashalg,
+					     ACVP_CIPHERTYPE_HASH,
+					     &tmp->cipher_aux));
+	tmp->prereqs = pbkdf->prereqvals;
+	tmp->prereq_num = pbkdf->prereqvals_num;
+	tmp->keylen[0] = DEF_ALG_ZERO_VALUE;
 
 out:
 	return ret;
@@ -57,7 +83,7 @@ int acvp_req_set_algo_pbkdf(const struct def_algo_pbkdf *pbkdf,
 
 	CKINT(acvp_req_add_revision(entry, "1.0"));
 
-	CKINT(acvp_req_set_prereq_pbkdf(pbkdf, entry, false));
+	CKINT(acvp_req_set_prereq_pbkdf(pbkdf, NULL, entry, false));
 
 	caps_array = json_object_new_array();
 	CKNULL(caps_array, -ENOMEM);
